@@ -13,6 +13,7 @@ import copy
 import matplotlib.pyplot as plt
 import ibm_boto3
 from botocore.client import Config
+import statsmodels.api as sm
 
 #-----------------------------
 def Read_Problem_data(model_id,icos_client,bucketName,feature_data_original,feature_data_normal):
@@ -53,8 +54,8 @@ def Read_Problem_data(model_id,icos_client,bucketName,feature_data_original,feat
     #print(PSN_WRI_dict)
     print('Data_PSN_for_all_list:',len(Data_PSN_for_all_list))
     print('Problem point (weld_record_index) list = ',len(Data_WRI_for_all_list))
-    dftemp= DR_file_df.loc[lambda x: (x.PSN.isin(Data_PSN_for_all_list)) & (x.WeldID == weld_id),:]
-    print(dftemp)
+    validation_report_df= DR_file_df.loc[lambda x: (x.PSN.isin(Data_PSN_for_all_list)) & (x.WeldID == weld_id),:]
+    print(validation_report_df)
     #
     # remove the defect that belongs to normal data
     #
@@ -85,12 +86,16 @@ def Read_Problem_data(model_id,icos_client,bucketName,feature_data_original,feat
                     'Data_WRI_for_all_list':Data_WRI_for_all_list,
                     'Data_PSN_for_all_list':Data_PSN_for_all_list,
                     'validate_df':validate_df,
-                    'validation_wo_normal_df':validation_wo_normal_df
+                    'validation_wo_normal_df':validation_wo_normal_df,
+                    'validation_report_df':validation_report_df
                    }
     return Problem_object
 
 
 def read_feature_data(model_id,feature_file_base_list,icos_client,bucketName):
+    COS_folder_source= 'Production_Lincoln_features_data/' 
+    COS_folder_TSA_target =  'Production_Lincoln_TSA_features_data/'
+
     #
     # read all feature data set from the file list
     #
@@ -101,6 +106,7 @@ def read_feature_data(model_id,feature_file_base_list,icos_client,bucketName):
     toolID =  'tool'+model_id.split('_tool')[1]
     
     for feature_file_base in feature_file_base_list:
+        # Read feature data
         try:
             feature_file_name = feature_file_base.replace('toolX',toolID)
             inputFile_date = feature_file_base.split('_LincolnFANUC_')[1].split('_welding_')[0]
@@ -110,15 +116,6 @@ def read_feature_data(model_id,feature_file_base_list,icos_client,bucketName):
             if not hasattr(body, "__iter__"): body.__iter__ = types.MethodType( __iter__, body )
             feature_data = pd.read_csv(body)
             feature_data_original = pd.concat([feature_data_original, feature_data])
-            # Read normal data
-            normal_file_name = feature_file_name[:-4] + '_normal.csv'
-            print(normal_file_name)
-            body = icos_client.get_object(Bucket=bucketName,Key=normal_file_name)['Body']
-            # add missing __iter__ method, so pandas accepts body as file-like object
-            if not hasattr(body, "__iter__"): body.__iter__ = types.MethodType( __iter__, body )
-            normal_data = pd.read_csv(body)
-            feature_data_normal = pd.concat([feature_data_normal, normal_data])
-
         except Exception as e:
             # Just print(e) is cleaner and more likely what you want,
             # but if you insist on printing message specifically whenever possible...
@@ -127,15 +124,35 @@ def read_feature_data(model_id,feature_file_base_list,icos_client,bucketName):
                 print(e.message)
             else:
                 print(e)
+                
+        # Read normal data
+        try:
+            normal_file_name = feature_file_name[:-4] + '_normal.csv'
+            normal_file_name = normal_file_name.replace(COS_folder_TSA_target,COS_folder_source)
+            print(normal_file_name)
+            body = icos_client.get_object(Bucket=bucketName,Key=normal_file_name)['Body']
+            # add missing __iter__ method, so pandas accepts body as file-like object
+            if not hasattr(body, "__iter__"): body.__iter__ = types.MethodType( __iter__, body )
+            normal_data = pd.read_csv(body)
+            feature_data_normal = pd.concat([feature_data_normal, normal_data])
+        except Exception as e:
+            # Just print(e) is cleaner and more likely what you want,
+            # but if you insist on printing message specifically whenever possible...
+            print('###### Exception, feature_data file might not exist in COS:' + normal_file_name)
+            if hasattr(e, 'message'):
+                print(e.message)
+            else:
+                print(e)
+                
 
-    feature_data_original   = feature_data_original.reset_index(drop = True)
-    feature_data_normal = feature_data_normal.reset_index(drop = True)
+    feature_data_original = feature_data_original.reset_index(drop = True)
+    feature_data_normal   = feature_data_normal.reset_index(drop = True)
 
     print('feature_data_original=',feature_data_original.shape)
     print('feature_data_normal=',feature_data_normal.shape)
     return feature_data_original,feature_data_normal
 
-
+#--------------------------------------------------------------------
 def export_training_validation_to_COS(model_id,icos_client,bucketName_model,feature_data_original,feature_data_normal,validate_df,validation_wo_normal_df,dateRange='',localPath='./Production_Anomaly_data/'):
     # Note: using model_id for file name in COS
     # Train
@@ -176,10 +193,13 @@ def export_training_validation_to_COS(model_id,icos_client,bucketName_model,feat
     print('write to COS:'+csv_fileName)
     return
 
-def read_training_validation_from_COS(model_id,icos_client,bucketName,bucketName_model,testData_date,dateRange=''):
+def read_training_validation_from_COS(model_id,icos_client,bucketName,bucketName_model,testData_date_list,dateRange=''):
     #
     #############################################################################################    
     # Training data
+    #COS_folder_source= 'Production_Lincoln_features_data/' # original
+    COS_folder_source=  'Production_Lincoln_TSA_features_data/' # TSA
+    #
     csv_fileName=model_id+'/Train_'+model_id+dateRange+'.csv'
     body = icos_client.get_object(Bucket=bucketName_model,Key=csv_fileName)['Body']
     # add missing __iter__ method, so pandas accepts body as file-like object
@@ -216,28 +236,45 @@ def read_training_validation_from_COS(model_id,icos_client,bucketName,bucketName
     #
     # === testDate date
     #
-    #testData_date = '2020-08-24'
+    #testData_date_list = ['2020-08-24','2020-08-25']
     # 
     deviceId = model_id.split('_weld')[0] # Formet_FR4_STA60_LH_R1_weld32_toolB
     weldId_with_toolID =  model_id.split('_weld')[1]
-    feature_file_name = 'Production_Lincoln_features_data/'+deviceId+'_LincolnFANUC_'+testData_date+'_welding_stable_data_weldid_'+weldId_with_toolID+'_feature.csv'
-    body = icos_client.get_object(Bucket=bucketName,Key=feature_file_name)['Body']
-    # add missing __iter__ method, so pandas accepts body as file-like object
-    if not hasattr(body, "__iter__"): body.__iter__ = types.MethodType( __iter__, body )
-    testData1 = pd.read_csv(body)
+    # get test data
+    testData_join = pd.DataFrame()
+    for weldDay in testData_date_list:
+        feature_file_name = COS_folder_source+deviceId+'_LincolnFANUC_'+weldDay+'_welding_stable_data_weldid_'+weldId_with_toolID+'_feature.csv'
+        body = icos_client.get_object(Bucket=bucketName,Key=feature_file_name)['Body']
+        # add missing __iter__ method, so pandas accepts body as file-like object
+        if not hasattr(body, "__iter__"): body.__iter__ = types.MethodType( __iter__, body )
+        testData1 = pd.read_csv(body)
+        testData_join = pd.concat([testData_join, testData1])
+        
+    if ('current_rms_min_resi' in  testData_join.columns):
+        pass
+    else:
+        testData_join   = add_TSA_trend_and_residual(testData_join)
+
+#    testData_join = pd.concat([testData_join,validate_df.loc[validate_df.label == 1]], sort=True).reset_index(drop=True)
+    testData_join = pd.concat([testData_join,validate_df], sort=True).reset_index(drop=True)
+    testData_join = testData_join.drop(['label'],axis = 1)
+    testData = testData_join.drop_duplicates(subset ="weld_record_index",keep='first',inplace=False)
+    #add TSA feature if not exist
 
-    testData1 = pd.concat([testData1,validate_df.loc[validate_df.label == 1]]).reset_index(drop=True)
-    testData1 = testData1.drop(['label'],axis = 1)
-    testData = testData1.drop_duplicates(subset ="weld_record_index",keep='first',inplace=False)
     testData = testData.sort_values(['weld_record_index'],inplace=False)
-    testData = testData.reset_index(drop=True)
+    testData = testData.reset_index(drop=True)    
     print ('test Data=',testData.shape)
+    
     return feature_data_original,feature_data_normal,validate_df,validation_wo_normal_df,testData
 #----------------------------------------------------------------
 #
 # add anomaly score as one column of feature csv file, and save it to COS
 #----------------------------------------------------------------
-def print_and_plot_anomaly_score(model_name,df_join_TF,Problem_object,anomaly_threshold,anomaly_score,percentile_point_95,plot_anomaly_score='YES'):
+def print_and_plot_anomaly_score(model_id,model_name,df_join_TF,Problem_object,anomaly_threshold,anomaly_score,percentile_point_95,plot_anomaly_score='YES'):
+    deviceId = model_id.split('_weld')[0] # Formet_FR4_STA60_LH_R1_weld32_toolB
+    weldId_with_toolID =  model_id.split('_weld')[1]
+    weld_id = int(weldId_with_toolID.split('_')[0])
+
     # get from problem report:
     DR_file_df = Problem_object['DR_file_df']
     DR_PSN_red_list = Problem_object['DR_PSN_red_list']
@@ -262,7 +299,10 @@ def print_and_plot_anomaly_score(model_name,df_join_TF,Problem_object,anomaly_th
     # plot anomaly score
     if (plot_anomaly_score == 'YES'):
         plt.figure(figsize=(20,4))
-        plt.plot(anomaly_score)
+        Y = anomaly_score.loc[: , 0]
+        X = list(range(len(Y)))
+        plt.scatter(X, Y) 
+        #plt.plot(anomaly_score)
         plt.title('Anomaly Score of each Weld based on Model ' + model_name)
         plt.xlabel('Observation')
         plt.ylabel('Anomaly Score')
@@ -281,24 +321,37 @@ def print_and_plot_anomaly_score(model_name,df_join_TF,Problem_object,anomaly_th
     
     
     # print
-
-    num_anomaly_list_PSN = [PSN_WRI_dict[x] for x in anomaly_above_threshold_join_defects]
-    num_anomaly_df = DR_file_df.loc[lambda x: (x.PSN.isin(num_anomaly_list_PSN)),:]
     df_join_PSN_TF = df_join_TF.rename(columns={"LPSN": "PSN"})
+    # abnormal points
+    num_anomaly_list_PSN = [PSN_WRI_dict[x] for x in anomaly_above_threshold_join_defects]
+    num_anomaly_df = DR_file_df.loc[lambda x: (x.PSN.isin(num_anomaly_list_PSN)) & (x.WeldID == weld_id),:]
+
     num_anomaly_df = num_anomaly_df.merge(df_join_PSN_TF[['PSN','anomaly_score']],on = ['PSN'], how = 'left')
     num_anomaly_df = num_anomaly_df.drop_duplicates(subset ="PSN",keep='first',inplace=False)
     num_anomaly_threshold = num_anomaly_df.shape[0]
-    #
+
+    # normal points
+    num_green_list_PSN = [PSN_WRI_dict[x] for x in normal_green_list]
+    num_green_df = DR_file_df.loc[lambda x: (x.PSN.isin(num_green_list_PSN)) & (x.WeldID == weld_id),:]
+
+    num_green_df = num_green_df.merge(df_join_PSN_TF[['PSN','anomaly_score']],on = ['PSN'], how = 'left')
+    num_green_df = num_green_df.drop_duplicates(subset ="PSN",keep='first',inplace=False)
+    
+    
     print('=========number of anomaly =============== \n percentile_95:',num_anomaly_95,', bad validation num above threshold:',num_anomaly_threshold)
     print('---------number of normal ---------------- \n num_normal_green_list:',num_normal_green_list,', total num_normal_below_threshold:',num_normal_below_threshold)
     print(num_anomaly_df)
+    print(num_green_df)
     return num_anomaly_95,num_anomaly_threshold
 
 #----------- anomaly score -----------------------
-def get_anomaly_score(model_id,icos_client,bucketName,pipeline,selected_features,testData_date,localPath='./Production_Anomaly_data/'):
+def get_anomaly_score(model_id,icos_client,bucketName,pipeline,selected_features,testData_date,COS_folder_source='Production_Lincoln_TSA_features_data/',localPath='./Production_Anomaly_data/'):
+    COS_folder_source= 'Production_Lincoln_features_data/' # original
+    #COS_folder_source=  'Production_Lincoln_TSA_features_data/' # TSA
+
     deviceId = model_id.split('_weld')[0] # Formet_FR4_STA60_LH_R1_weld32_toolB
     weldId_with_toolID =  model_id.split('_weld')[1]
-    feature_file_name = 'Production_Lincoln_features_data/'+deviceId+'_LincolnFANUC_'+testData_date+'_welding_stable_data_weldid_'+weldId_with_toolID+'_feature.csv'
+    feature_file_name = COS_folder_source+deviceId+'_LincolnFANUC_'+testData_date+'_welding_stable_data_weldid_'+weldId_with_toolID+'_feature.csv'
     body = icos_client.get_object(Bucket=bucketName,Key=feature_file_name)['Body']
     # add missing __iter__ method, so pandas accepts body as file-like object
     if not hasattr(body, "__iter__"): body.__iter__ = types.MethodType( __iter__, body )
@@ -325,3 +378,27 @@ def get_anomaly_score(model_id,icos_client,bucketName,pipeline,selected_features
     icos_client.upload_file(Filename=localfileName,Bucket=bucketName,Key=csv_fileName)
     print('write to COS:'+csv_fileName)
     return feature_data
+
+#-----------------------------------------
+#------------------add TSA components
+def add_TSA_trend_and_residual(feature_data,freqInput=125):
+    # feature list
+    features_select_TSA = ['current_rms_min','current_rms_max', 'current_rms_mean','current_rms_std','current_cd_max',
+                           'voltage_rms_min','voltage_rms_max', 'voltage_rms_mean','voltage_rms_std','voltage_cd_max',
+                           'motor_current_rms_min','motor_current_rms_max','motor_current_rms_mean','motor_current_rms_std',
+                           'motor_current_rms_skew','wire_feed_speed_rms_std',
+                           'power_rms_min','power_rms_max','power_rms_mean','power_rms_std', 'std_power','max_energy']
+    
+    # loop the features
+    for tsa_feature in features_select_TSA:
+        # original data
+        res = sm.tsa.seasonal_decompose(feature_data[tsa_feature], freq=freqInput)
+        df_resi = pd.DataFrame(res.resid).abs()
+        df_trend = pd.DataFrame(res.trend).abs()
+        feature_data[tsa_feature+'_resi'] = df_resi
+        feature_data[tsa_feature+'_trend'] = df_trend
+    #
+    feature_data = feature_data.dropna(how = 'any', axis = 0).reset_index(drop =True)
+
+    return feature_data
+ 
